#ifndef _CAVETV
#define _CAVETV

/*
 *
 * This is part of the ANL/MCS/FL CAVEav distribution.
 * written by Ivan R. Judson
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>

#include "FLSharedMemory.h"
#include "CAVSharedObject.h"
#include "CAVImagePool.h"

#define CAVEAVMAXTEXTURES 1

/** This is the CAVEtv base class from which all CAVEtv objects are
  derived.  The CAVEtv is an object that displays video streams.  
  */

class CAVtv : public CAVSharedObject {
public:
  /** @name Methods */
  //@{
    
  /// The default constructor takes three arguments the x, y, and z position of the CAVEtv in CAVE space.
  
  CAVtv(float x = 0.0, float y = 0.0, float z = 0.0);
  
  /// The default destructor takes no arguments and is virtual.

  virtual ~CAVtv();

  /// This accessor sets the width of the tv object

  void SetTVWidtth(int _width) { _tv_width = _width; }
  
  /// This accessor sets the height of the tv object
  
  void SetTVHeight(int _height) { _tv_height = _height; }

  /// This accessor sets the width of the video pane on the TV
  
  void SetVideoWidth(int _width) { _video_width = _width; }

  /// This accessor sets the height of the video pane on the TV
  
  void SetVideoHeight(int _height) { _video_height = _height; }

  /// This accessor sets the depth of the video pane on the TV

  void SetVideoDepth(int _depth) { _video_depth = _depth; }

  /// This accessor sets the color and alpha value of the CAVEtv
  
  void SetColor(float r, float g, float b, float a) {
    _red = r; _green = g; _blue = b; _alpha = a;
  }

  /// This accessor sets the color of the CAVEtv
  
  void SetColor(float r, float g, float b) {
    _red = r; _green = g; _blue = b;
  }

  /// This accessor sets the x, y, and z position of the CAVEtv

  void SetPosition(float x, float y, float z) {
    _px = x; _py = y; _pz = z;
  }

  /// This function sets how fast the frames are read in
	
  void SetFrameRate(int frate) {
    _frate = frate;
  }

  /// This function gets the next power of two
  int GetPowTwo(int val) {
    int vout = 1;
    
    while(vout < val) {
      vout *= 2;
    }
    
    return vout;
  }
    
  /// This function is where tv local initialization takes place
	
  virtual void Init(void);

  /// This function is where the frame is read in and moved into tv local space
  
  virtual void GetPix(void);

  /// This function is where the frame is placed on the CAVEtv

  virtual void Process(void);

  /// This function draws the tv object in the CAVE scene

  virtual void Draw(void);
  //@}

protected:

  /**@name Data */
  //@{
  
  /// The width of the tv object

  int _tv_width;
  
  /// The height of the tv object

  int _tv_height;

  /// The width of the video buffer

  int _video_width;
  
  /// The height of the video buffer

  int _video_height;
  
  /// The depth of the video buffer

  int _video_depth;

  /// The size of the video buffer

  int _video_size;

  /// The red value for the tv

  float _red;
  
  /// The green value for the tv

  float _green;
  
  /// The blue value for the tv
  
  float _blue;

  /// The alpha value for the tv

  float _alpha;

  /// The x position of the tv

  float _px;

  /// the y position of the tv

  float _py;

  /// The z position of the tv

  float _pz;

  /// The texture ID

  GLuint _texids[CAVEAVMAXTEXTURES];

  /// Ring Buffer of image descriptors

  CAVImagePool *_imagepool;

  /// Index of last image used from Ring Buffer

  int _lastimage;

  /// Serial number of last image used
  int _lastserial;

  /// Current image size (height * width * depth)

  int _fb_size;

  /// Frame Counter

  int _fc;
  
  /// Frame rate

  int _frate;

  /// This is the size in texels of the image for the S val

  GLfloat _s_coord;

  /// This is the size in texels of the image for the T val

  GLfloat _t_coord;

  //@}
};

#endif

  
  
