#ifndef _NETCAVETV
#define _NETCAVETV

/*
 *
 * This is part of the ANL/MCS/FL CAVEav distribution.
 * written by Ivan R. Judson
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "CAVtv.h"
#include "TexturedPlane.h"

/** This is the derived class netCAVEtv, which takes frames from a 
  sequence of rgb format files, and places them on a video pane on a 
  tv object in a CAVE scene.
  */

class netCAVtv : public CAVtv {

public:
  /**@name Methods */
  //@{
    
  /// The constructor takes seven arguments, the x, y, z position of the TV the file basename to open foreach frame, and the width, height and depth of the frames.
	  
  netCAVtv(char *hostname = NULL, int port = 0,
	     float x = 0.0, float y = 0.0, float z = 0.0);

  /// The destructor must cleanup the imagepool

  ~netCAVtv(void);

  /// The init function

  void Init(void);

  /// This function reads the frame in

  void GetPix(void);

  /// This is the inherited draw function

  void Draw(void);

  /// This is the inherited process function

  void Process(void);

  //@}
protected:  
  
  /// Private Init function

  void pInit(void);

private:
  /**@name Data */
  //@{

  /// Shared memory key

  key_t _shmkey;

  /// Shared Memory Object
  
  FLSharedMemory *_shmem;
  
  /// Hostname

  char *_host;
  
  /// Port

  int _port;

  /// vic pid
  
  int _pid;
  
  /// camera config file name

  char *_fn;

  /// Texture pane

  TexturedPlane *tp_;

  /// Debugging data
  
  unsigned long *debugData_;

  /// Gross hack

  int initted_;

  //@}
};

#endif
