#include "TexturedPlane.h"

#include <GL/glu.h>
#include "cave_ogl.h"

#define TEX_EXT
//#define DEBUG

TexturedPlane::TexturedPlane() : format_(GL_RGB)
{
    int i;
    
    textureSerial_ = 0;
    for (i = 0; i < CAVE_MAX_WALLS; i++)
    {
	firstBuild_[i] = 1;
	wallSerial_[i] = 0;
    }

    lock_ = CAVENewLock();

    textureWidth_ = textureHeight_ = 0;
    textureData_ = 0;
}

TexturedPlane::TexturedPlane(int width, int height, int depth)
{
    int i;
    
    textureSerial_ = 0;
    for (i = 0; i < CAVE_MAX_WALLS; i++)
    {
	firstBuild_[i] = 1;
	wallSerial_[i] = 0;
    }

    textureWidth_ = width;
    textureHeight_ = height;
    textureDepth_ = depth;

    nextpowtwowidth_ = FLUtils::getNextPowTwo(width);
    nextpowtwoheight_ = FLUtils::getNextPowTwo(height);

    lock_ = CAVENewLock();

    textureData_ = 0;
}

TexturedPlane::~TexturedPlane(void)
{

}

void setFormat(int format)
{

}

void TexturedPlane::setTexture(int width, int height, int depth,
			       unsigned char *data, GLenum format)
{
  CAVESetWriteLock(lock_);
  format_ = format;
  textureWidth_ = width;
  textureHeight_ = height;
  textureDepth_ = depth;
  textureData_ = data;

  int newP2w = FLUtils::getNextPowTwo(width);
  int newP2h = FLUtils::getNextPowTwo(height);

  if (newP2w != nextpowtwowidth_ ||
      newP2h != nextpowtwoheight_)
  {
      int i;
      
      nextpowtwowidth_ = FLUtils::getNextPowTwo(width);
      nextpowtwoheight_ = FLUtils::getNextPowTwo(height);
      for (i = 0; i < CAVE_MAX_WALLS; i++)
	  firstBuild_[i] = 1;
  }

#ifdef DEBUG
  fprintf(stderr, "OX: %d OY: %d NX: %d NY: %d\n", textureWidth_,
	  textureHeight_, nextpowtwowidth_, nextpowtwoheight_);
  fprintf(stderr, "depth=%d format=%d\n", depth, format);
#endif
  textureSerial_++;
  CAVEUnsetWriteLock(lock_);
}

void TexturedPlane::appInit()
{
    int myWall = getWallIndex();

    dlIndex_[myWall] = glGenLists(1);

    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
}

void TexturedPlane::frame()
{
    int myWall = getWallIndex();

    CAVESetReadLock(lock_);

    if (textureSerial_ > wallSerial_[myWall])
    {
	if (firstBuild_[myWall])
	{
	    buildWall(myWall);
	    firstBuild_[myWall] = 0;
	}
	else
	    rebuildWall(myWall);
    }

    CAVEUnsetReadLock(lock_);
}


#ifdef TEX_EXT
void TexturedPlane::buildWall(int myWall)
{
    printf("Build: format=%d\n", format_);
  glTexImage2D(GL_TEXTURE_2D, 0, 4, nextpowtwowidth_, nextpowtwoheight_,
	       0, format_, GL_UNSIGNED_BYTE, (GLvoid *) NULL);
  
  
  rebuildWall(myWall);
}

void TexturedPlane::rebuildWall(int myWall)
{
    float smin = 0.0,  tmin = 0.0;
    float smax, tmax;

#ifdef DEBUG
    printf("in rebuild: tex %d %d next %d %d\n",
	   textureWidth_, textureHeight_,
	   nextpowtwowidth_, nextpowtwoheight_);
#endif

    smax = (float)textureWidth_ / (float)nextpowtwowidth_;
    tmax = (float)textureHeight_ / (float)nextpowtwoheight_;

//    smax = 352.0 / 512.0;
//    tmax = 288.0 / 512.0;

//    smax = 0.6;
//    tmax = 0.6;

//  tmin = 1 - tmax;
//  tmax = 1.0;

#ifdef DEBUG
  printf("smin=%f tmin=%f\n", smin, tmin);
  printf("smax=%f tmax=%f\n", smax, tmax);
#endif
  
  glTexSubImage2DEXT(GL_TEXTURE_2D, 0, 0, 0,
		     nextpowtwowidth_, nextpowtwoheight_, format_,
		     GL_UNSIGNED_BYTE, (GLvoid *)textureData_);
  
  glNewList(dlIndex_[myWall], GL_COMPILE);
  
  glEnable(GL_TEXTURE_2D);

  glBegin(GL_QUADS);
  {
    glTexCoord2f(smin, tmin);
    glVertex2f(-0.5, -0.5);
    glTexCoord2f(smin, tmax);
    glVertex2f(-0.5, 0.5);
    glTexCoord2f(smax, tmax);
    glVertex2f(0.5, 0.5);
    glTexCoord2f(smax, tmin);
    glVertex2f(0.5, -0.5);
  }
  glEnd();

  glDisable(GL_TEXTURE_2D);

  glBegin(GL_QUADS);
  {
    glColor3f(0.0, 1.0, 1.0);
    glVertex3f(-0.6, -0.6, -0.05);
    glVertex3f(-0.6, 0.6, -0.05);
    glVertex3f(0.6, 0.6, -0.05);
    glVertex3f(0.6, -0.6, -0.05);
  }
  glEnd();

  glEndList();
  
  wallSerial_[myWall] = textureSerial_;
}

#else /* TEX_EXT */

void TexturedPlane::buildWall(int myWall)
{
  float smax = (float)textureWidth_ / (float)nextpowtwowidth_;
  float tmax = (float)textureHeight_ / (float)nextpowtwoheight_;

  glTexImage2D(GL_TEXTURE_2D, 0, 3, nextpowtwowidth_, nextpowtwoheight_,
	       0, format_, GL_UNSIGNED_BYTE, (GLvoid *)textureData_);
  glNewList(dlIndex_[myWall], GL_COMPILE);
  glEnable(GL_TEXTURE_2D);

  glBegin(GL_QUADS);
  {
    glTexCoord2f(0.0, 0.0);
    glVertex2f(-0.5, -0.5);
    glTexCoord2f(0.0, tmax);
    glVertex2f(-0.5, 0.5);
    glTexCoord2f(smax, tmax);
    glVertex2f(0.5, 0.5);
    glTexCoord2f(smax, 0.0);
    glVertex2f(0.5, -0.5);
  }
  glEnd();

  glDisable(GL_TEXTURE_2D);

  glBegin(GL_QUADS);
  {
    glColor3f(1.0, 0.0, 0.0);
    glVertex3f(-0.5, -0.5, -0.1);
    glVertex3f(-0.5, 0.5, -0.1);
    glVertex3f(0.5, 0.5, -0.1);
    glVertex3f(0.5, -0.5, -0.1);
  }
  glEnd();

  glEndList();
  
  wallSerial_[myWall] = textureSerial_;
}

void TexturedPlane::rebuildWall(int myWall)
{
    buildWall(myWall);
}

#endif /* TEX_EXT */

void TexturedPlane::draw()
{
    glPushMatrix();
    {
      glEnable(GL_COLOR_MATERIAL);
//      glEnable(GL_TEXTURE_2D);

      glCallList(dlIndex_[getWallIndex()]);

//      glDisable(GL_TEXTURE_2D);
      glDisable(GL_COLOR_MATERIAL);
    }
    glPopMatrix();
}
