/*
 *
 * This is part of the ANL/MCS/FL CAVEav distribution.
 * written by Ivan R. Judson
 *
 */

#include <fileCAVtv.h>

#undef DEBUG_TEX

fileCAVtv::fileCAVtv(char *fn, int width, int height, int depth,
		     float x, float y, float z) : CAVtv(x, y, z)
{
#ifdef DEBUG
  fprintf(stderr, "Entering fileCAVtv constructor.\n");
#endif
  _px = x; _py = y; _pz = z;
  _video_width = width; _video_height = height; _video_depth = depth;
  _video_size = _video_width * _video_height * _video_depth;
  _fnbase = fn;

  initted_ = 0;

  tp_ = new TexturedPlane(width, height, depth);

#ifdef DEBUG_TEX
  debugData_ = new unsigned long[_video_size];
  for(int j = 0; j < _video_size/_video_depth; j++)
    debugData_[j] = (unsigned long)0xff00ff00;
#endif

  _tv_width = 3.0;
  _tv_height = 2.0;

  _fc = 0;
  _imagepool = new CAVImagePool(width, height, depth, 10);

#ifdef DEBUG
  fprintf(stderr, "Exiting filCAVtv constructor.\n");
#endif
}

fileCAVtv::~fileCAVtv()
{
  delete _imagepool;
  delete tp_;
}

void fileCAVtv::Init(void)
{
}

void fileCAVtv::pInit(void)
{
#ifndef DEBUG_TEX
  CAVImageDescriptor *image;
  int nextIndex;

  nextIndex = (_imagepool->_nextImage + 1) % _imagepool->_nImages;
  image = &(_imagepool->_images[nextIndex]);
#endif
  
  if(!initted_)
    {
      initted_ = 1;

      tp_->appInit();

#ifndef DEBUG_TEX
      GetPix();
      tp_->setTexture(_video_width, _video_height, _video_depth,
		      (unsigned char *)image->GetData());
#else
      tp_->setTexture(_video_width, _video_height, _video_depth,
		      (unsigned char *)debugData_);
#endif  
    }

}

void fileCAVtv::Draw(void)
{
  // This is a unit cube therefore its eight corners are:

  static float flb[3] = { -0.5,  -0.5, 0.0 };  //  flb = front left bottom
  static float flt[3] = { -0.5,   0.5, 0.0 };  //  flt = front left top
  static float frt[3] = {  0.5,   0.5, 0.0 };  //  frt = front right top
  static float frb[3] = {  0.5,  -0.5, 0.0 };  //  frb = front right bottom
  static float blt[3] = { -0.5,   0.5, -1.0 };  //  blt = back left top
  static float blb[3] = { -0.5,  -0.5, -1.0 };  //  blb = back left bottom
  static float brt[3] = {  0.5,   0.5, -1.0 };  //  brt = back right top
  static float brb[3] = {  0.5,  -0.5, -1.0 };  //  brb = back right bottom

  Process();

  glPushMatrix();
  {
     glTranslatef(_px, _py, _pz);

     glScalef(_tv_width-0.2, _tv_height-0.2, 1.0);

     tp_->draw();
  }
  glPopMatrix();

  // Draw the TV
   glPushMatrix();
   {
     glTranslatef(_px, _py, _pz);

     glScalef(_tv_width, _tv_height, 1.0);

     glColor3f(_red, _green, _blue);
     glBegin(GL_QUADS);
     {
       // top
       glVertex3fv(flt);
       glVertex3fv(frt);
       glVertex3fv(brt);
       glVertex3fv(blt);

       // bottom
       glVertex3fv(flb);
       glVertex3fv(frb);
       glVertex3fv(brb);
       glVertex3fv(blb);

       // back
       glVertex3fv(blb);
       glVertex3fv(brb);
       glVertex3fv(brt);
       glVertex3fv(blt);

       // left
       glVertex3fv(flb);
       glVertex3fv(blb);
       glVertex3fv(blt);
       glVertex3fv(flt);

       // right
       glVertex3fv(frb);
       glVertex3fv(brb);
       glVertex3fv(brt);
       glVertex3fv(frt);
     }
     glEnd();
   }
   glPopMatrix();
}

int fileCAVtv::loadFile(char *filename)
{
  FILE *fp;
  CAVImageDescriptor *image;
  int nextIndex;

  nextIndex = (_imagepool->_nextImage + 1) % _imagepool->_nImages;
  image = &(_imagepool->_images[nextIndex]);

  if(access(filename, R_OK) < 0) 
    {
      perror("\nNo file");
      return 0;
    }
  else if((fp = fopen(filename, "r")) != NULL)
    {
      if(fread((void *)image->GetData(), 
	       _video_width * _video_height * _video_depth,
	       1, fp))
	{
	  fclose(fp);
#ifdef DEBUG
	  fprintf(stderr, "closed\n");
	  return 1;
#endif
	} else {

	  perror("Can't read frame");
	  return 0;
	}
    } else {
      perror("Can't open file");
      return 0;
    }
  return 1;
}

void fileCAVtv::GetPix(void)
{
  char fn[128];

  sprintf(fn, "%s%d.raw", _fnbase, _fc++);

#ifdef DEBUG
  fprintf(stderr, "Opening %s to read from...", fn);
#endif

  if(!loadFile(fn))
    _fc = 0;
}

void fileCAVtv::Process(void)
{
  CAVImageDescriptor *image;
  int nextIndex;
  
  // Get things pointed at the right memory and retrive values
  nextIndex = (_imagepool->_nextImage + 1) % _imagepool->_nImages;
  image = &(_imagepool->_images[nextIndex]);

  pInit();

  if(CAVEEye == CAVE_LEFT_EYE)
    tp_->frame();

  if(!(_fc % _frate)) 
    {
#ifndef DEBUG_TEX
      GetPix();
 
      tp_->setTexture(_video_width, _video_height, _video_depth,
		      (unsigned char *)image->GetData());
#else
      tp_->setTexture(_video_width, _video_height, _video_depth,
		      (unsigned char *)debugData_);
#endif
    }
}
