/*
 *
 * This is part of the ANL/MCS/FL CAVEav distribution.
 * written by Ivan R. Judson
 *
 */

#include <netCAVtv.h>

#include <stdio.h>
#include <sys/types.h>
#include <signal.h>
#include <time.h>

#define CONFDATA " \
 \noption add vic.muteNewSources true startupFile \
 \n \
 \n\nproc user_hook {} { \
 \n\tglobal outputDeviceList \
 \n\tforeach v $outputDeviceList { \
 \n\t\tif { [$v nickname] == \"shmem\" } { \
 \n\t\t\t$v id %d \n\t\t} \n\t}\n} "
// \nproc check_for_src { v } { \
// \n\tglobal sources_seen \
// \n\tforeach src [session active] { \
// \n\t\tif [ info exists sources_seen($src) ] { \
// \n\t\t} else { \
// 	    puts "new source $src"
// 	    if { $src == [srctab local] } {
// 		puts "local source"
// 		set sources_seen($src) 1
// 	    } elseif { [$src sdes cname] == "" } {
// 		puts "no cname for $src"
// 	    } else {
// 		new_source $src $v
// 		set sources_seen($src) 1
// 	    }
// 	}
//     }

//     after 500 check_for_src $v
// }

// proc new_source { src dev } {
//     global src_nickname mutebutton
//     set cname [$src sdes cname]
//     set addr [ $src addr ]
//     puts "new source $src cname $cname $addr"

//     set want [option get . wantSource Vic]
//     if { $want == $cname } {
//         puts "Got what we wanted"
// 	set was $mutebutton($src)
// 	set mutebutton($src) 0
// 	set is $mutebutton($src)
// 	switcher_set $dev $src
//     }
// }
// "

#undef DEBUG_TEX

netCAVtv::netCAVtv(char *hostname, int port, 
		     float x, float y, float z) : CAVtv(x, y, z)
{
#ifdef DEBUG
  fprintf(stderr, "Entering netCAVtv constructor.\n");
#endif
  time_t tloc;
  time(&tloc);
  srand48((long)tloc);

  while(_shmkey < 1)
    _shmkey = lrand48() % 32768;

#ifdef DEBUG
    fprintf(stderr, "Key: %d\n", _shmkey);
#endif

  _host = hostname;
  _port = port;

  _fn = tempnam("/tmp/","TV");

  _shmem = new FLSharedMemory(_shmkey, sizeof(CAVImagePool), 
                               0666);
  tp_ = NULL;

  _imagepool = (CAVImagePool *)_shmem->GetAddress();

  _imagepool->_nImages = 10;
  _imagepool->_totalPoolSize = sizeof(CAVImagePool);
      
  _px = x; _py = y; _pz = z;

  initted_ = 0;

#ifdef DEBUG_TEX
  debugData_ = new unsigned long[512*512*3];
  for(int j = 0; j < 512*512; j++)
    debugData_[j] = (unsigned long)0xff00ff00;
#endif

  _tv_width = 3.0;
  _tv_height = 2.0;

  _fc = 0;

#ifdef DEBUG
  fprintf(stderr, "Exiting filCAVtv constructor.\n");
#endif
}

netCAVtv::~netCAVtv()
{
  unlink(_fn);
  kill(_pid, SIGKILL);
  delete _shmem;
  delete _imagepool;
  delete tp_;
}

void netCAVtv::Init(void)
{
  FILE *fp;
  char cmdpath[256];
  char cmdargs[2][128];
  char vicconf[1024];
  char *envpath;
  
#ifdef DEBUG
  fprintf(stderr, "Entering netCAVcam Init function.\n");
#endif

  sprintf(cmdpath,"%s",  VICPATH);

  if((envpath = getenv("CAV_VIC")) != NULL)
    strcpy(cmdpath, envpath);

#ifdef DEBUG
  fprintf(stderr, "Opening vic configuration file...");
#endif

  if((fp = fopen(_fn, "w")) != NULL) 
    {
      sprintf(vicconf, CONFDATA, _shmem->GetID());
      fprintf(fp, "%s\n", vicconf);
      fclose(fp);
    } else {
      perror("Can't open file");
      CAVEExit();
    }

#ifdef DEBUG
  fprintf(stderr, "done.\nBuilding vic command line...");
#endif

  sprintf(cmdargs[0], "-u %s", _fn);
  sprintf(cmdargs[1], "%s/%d/h261/4", _host, _port);

#ifdef DEBUG
  fprintf(stderr, "done.\nForking vic process...");
#endif
  _pid = fork();

  if(!_pid)
    {
#ifdef DEBUG
      fprintf(stderr, "done.\nExecing vic...");
      fprintf(stderr, "%s %s %s\n", cmdpath, cmdargs[0], cmdargs[1]);
#endif
      if(execl(cmdpath, cmdpath, cmdargs[0], cmdargs[1], NULL) == -1) 
	{
	  perror("exec failed");
	  CAVEExit();
	}
#ifdef DEBUG
      fprintf(stderr, "done.\n");
#endif
    }

#ifdef DEBUG
  fprintf(stderr, "Successfully exiting netCAVcam Init function.\n");
#endif
}

void netCAVtv::pInit(void)
{
  CAVImageDescriptor *image;
  int nextIndex;

  nextIndex = (_imagepool->_nextImage + 1) % _imagepool->_nImages;
  image = &(_imagepool->_images[nextIndex]);

  if(tp_ == NULL)
    {
      tp_ = new TexturedPlane(image->GetWidth(), image->GetHeight(), 
			      image->GetDepth());
    }

  if(!initted_)
    {
      initted_ = 1;

      tp_->appInit();
      
#ifndef DEBUG_TEX
      GetPix();
      tp_->setTexture(image->GetWidth(), image->GetHeight(), image->GetDepth(),
		      (unsigned char *)image->GetData());
#else
      tp_->setTexture(_video_width, _video_height, _video_depth,
		      (unsigned char *)debugData_);
#endif  
    }
}

void netCAVtv::Draw(void)
{
  // This is a unit cube therefore its eight corners are:

  static float flb[3] = { -0.5,  -0.5, 0.0 };  //  flb = front left bottom
  static float flt[3] = { -0.5,   0.5, 0.0 };  //  flt = front left top
  static float frt[3] = {  0.5,   0.5, 0.0 };  //  frt = front right top
  static float frb[3] = {  0.5,  -0.5, 0.0 };  //  frb = front right bottom
  static float blt[3] = { -0.5,   0.5, -1.0 };  //  blt = back left top
  static float blb[3] = { -0.5,  -0.5, -1.0 };  //  blb = back left bottom
  static float brt[3] = {  0.5,   0.5, -1.0 };  //  brt = back right top
  static float brb[3] = {  0.5,  -0.5, -1.0 };  //  brb = back right bottom


  if(_imagepool->_clientActive)
    {
      pInit();
    }

  if(tp_ != NULL)
    {
      Process();
      
      glPushMatrix();
      {
	glRotatef(180.0, 0.0, 0.0, -1.0);
	glRotatef(180.0, 0.0, 1.0, 0.0);

	glTranslatef(_px, _py, _pz);
	
	glScalef(_tv_width-0.02, _tv_height-0.02, 1.0);
	
	tp_->draw();
      }
      glPopMatrix();
    }
  
  // Draw the TV
  glPushMatrix();
  {
    glTranslatef(_px, _py, _pz);
    
    glScalef(_tv_width, _tv_height, 1.0);
    
    glColor3f(_red, _green, _blue);
    glBegin(GL_QUADS);
    {
      // top
      glVertex3fv(flt);
      glVertex3fv(frt);
      glVertex3fv(brt);
      glVertex3fv(blt);
      
      // bottom
      glVertex3fv(flb);
      glVertex3fv(frb);
      glVertex3fv(brb);
      glVertex3fv(blb);
      
      // back
      glVertex3fv(blb);
      glVertex3fv(brb);
      glVertex3fv(brt);
      glVertex3fv(blt);
      
      // left
      glVertex3fv(flb);
      glVertex3fv(blb);
      glVertex3fv(blt);
      glVertex3fv(flt);
      
      // right
      glVertex3fv(frb);
      glVertex3fv(brb);
      glVertex3fv(brt);
      glVertex3fv(frt);
    }
    glEnd();
  }
  glPopMatrix();
}

void netCAVtv::GetPix(void)
{
}

void netCAVtv::Process(void)
{
  CAVImageDescriptor *image;
  int nextIndex;
  
  // Get things pointed at the right memory and retrive values
  nextIndex = (_imagepool->_nextImage + 1) % _imagepool->_nImages;
  image = &(_imagepool->_images[nextIndex]);

  if(CAVEEye == CAVE_LEFT_EYE)
    tp_->frame();
  
  if(!(_fc % _frate)) 
    {
#ifndef DEBUG_TEX
      GetPix();

#ifdef DEBUG
      fprintf(stderr, "X: %d Y: %d\n", image->GetWidth(), image->GetHeight());
#endif

      tp_->setTexture(image->GetWidth(), image->GetHeight(), image->GetDepth(),
		      (unsigned char *)image->GetData());
#else
      tp_->setTexture(_video_width, _video_height, _video_depth,
		      (unsigned char *)debugData_);
#endif
    }
}


